/***********************************************************************
ColorSafe 1.0 Beta - http://www.woven-media.com/

A Java 6 Swing application for color testing and conversions from RGB to
hexadecimal data formats and vice versa.

Copyright (C) 2008-2009, Brent Allen Parrish <info@woven-media.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
************************************************************************/
package com.woven_media.colorsafe;

// woven-media.com
import com.woven_media.colorsafe.WindowUtil;
import com.woven_media.colorsafe.Controller;

import java.net.URLClassLoader;
import java.util.ResourceBundle;
import java.awt.Graphics;
import java.awt.EventQueue;
import java.awt.Point;
import java.awt.Font;
import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JRootPane;
import javax.swing.SpringLayout;
import javax.swing.ImageIcon;
import javax.imageio.ImageIO;

/**
 * About class creates a borderless <code>About</code> box with background image which can run
 * as a stand-alone application.
 *
 * @author  Brent Allen Parrish
 * @version %I%, %G%
 * @since   1.0
 */
public class About extends JPanel
{
    final BufferedImage bimg;
    private static Controller controller;
    private static String title;
    private static JFrame host;
    private static Dimension boxSize;
    private static String imgBg;
    private static String resPath;
    private SpringLayout layout;
    private JFrame frame;
    private URLClassLoader ucl;

    /**
     * Class constructor sets <code>frame</code> member with reference from <code>host</code> window.
     * Initializes the <code>javax.swing.SpringLayout</code>, reads in the resource bundle and the
     * background image and assigns image to the <code>java.awt.image.BufferedImage</code> member.
     *
     * @param  f           <code>javax.swing.JFrame</code> reference for the parent window.
     * @throws IOException can be thrown from <code>javax.imageio.ImageIO#read</code>
     * @see                javax.imageio.ImageIO
     * @since              1.0
     */
    public About(JFrame f) throws IOException
    {
        frame = f;
        layout = new SpringLayout();
        setPreferredSize(getBoxSize());
        setLayout(layout);
        bimg = ImageIO.read(ucl.getSystemResource(resPath + imgBg));
        init();
    }

    /**
     * Instantiates components and adds listener for window close events. Adds
     * <code>close</code> button to the spring layout.
     *
     * @since 1.0
     */
    private void init()
    {
        Controller.loadIcon(frame);
        JButton closeBtn = new JButton("Close");
        closeBtn.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                if(frame != null)
                {
                    frame.setVisible(false);
                    frame.dispose();
                }
            }
        }); // End anonymous

        closeBtn.setFocusPainted(false);
        // Setup button layout     <<<< let's get this out of here so user can configure
        closeBtn.setSize(50, 50);
        closeBtn.setFont(new Font("Verdana", Font.PLAIN, 10));
        closeBtn.setBorder(null);  // How about a 2D button here? Needs to be configurable, too.
        int pad = 30;
        Dimension fSize = getPreferredSize();
        int xPos = (fSize.width - closeBtn.getWidth());
        int yPos = (fSize.height - closeBtn.getHeight()) + pad;

        layout.putConstraint(SpringLayout.WEST, closeBtn, xPos, SpringLayout.WEST, this);
        layout.putConstraint(SpringLayout.NORTH, closeBtn, yPos, SpringLayout.NORTH, this);
        add(closeBtn);
    }

    /**
     * Draws background image.
     *
     * @param g <code>java.awt.Graphics</code> object.
     * @since   1.0
     */
    protected void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        g.drawImage(bimg,getWidth()-bimg.getWidth(),getHeight()-bimg.getHeight(),null);
    }

    /**
     * Sets the <code>host</code> window reference statically.
     *
     * @param f static reference for the host window.
     * @since   1.0
     */
    public static void setHost(JFrame f)
    {
        host = f;
    }

    /**
     * Sets <code>Controller</code> instance for this <code>About</code> box.
     *
     * @param c static reference to the <code>Controller</code> instance.
     * @since   1.0
     */
    public static void setController(Controller c)
    {
        controller = c;
    }

    /**
     * Returns the <code>host</code> window reference.
     *
     * @return <code>javax.swing.JFrame</code> reference of the <code>host</code> window.
     * @since  1.0
     */
    public static JFrame getHost()
    {
        return host;
    }

    /**
     * Sets the <code>About</code> box window size statically for pre-configuration.
     *
     * @param w <code>int</code> indicating the desired width of the <code>About</code> box.
     * @param h <code>int</code> indicating the desired height of the <code>About</code> box.
     * @since   1.0
     */
    public static void setBoxSize(int w, int h)
    {
        boxSize = new Dimension(w, h);
    }

    /**
     * Returns the desired <code>About</code> box window size.
     *
     * @return <code>java.awt.Dimension</code> object
     * @since  1.0
     */
    public static Dimension getBoxSize()
    {
        return boxSize;
    }
    /**
     * Sets the <code>img</code> member to the string name of the icon image statically for
     * pre-configuration.
     *
     * @param bg <code>String</code> value representing background image string.
     * @since 1.0
     */
    public static void setBackgroundImageString(String bg)
    {
        imgBg = bg;
    }

    /**
     * Sets the default resource file path of this application.
     *
     * @param path <code>java.lang.String</code> value representing this application's resource file path.
     * @since      1.0
     */
    public static void setDefaultResourcePath(String path)
    {
        resPath = path;
    }

    /**
     * Sets the <code>com.woven_media.colorsafe.About</code> box title bar.
     *
     * @param t <code>java.lang.String</code> titlebar string.
     * @since   1.0
     */
    public static void setTitleBar(String t)
    {
        title = t;
    }

    /**
     * Returns the <code>com.woven_media.colorsafe.About</code> box titlebar string.
     *
     * @since         1.0
     */
    public static String getTitleBar()
    {
        return title;
    }

    /**
     * Application entry point that implements <code>java.lang.Runnable</code> anonymously.
     *
     * @param args string array of commandline arguments.
     * @see        java.lang.Runnable
     * @see        WindowUtil
     * @since      1.0
     */
    public static void main(String[] args)
    {
        EventQueue.invokeLater(new Runnable()
        {
            public void run()
            {
                try
                {
                    final JFrame f = new JFrame(About.getTitleBar());
                    f.setUndecorated(true);
                    f.getRootPane().setWindowDecorationStyle(JRootPane.NONE);
                    f.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
                    f.addWindowListener(new WindowAdapter()
                    {
                        public void windowDeactivated(WindowEvent e)
                        {
                            f.setVisible(false);
                            f.dispose();
                        }

                    }); // End anonymous
                    try
                    {
                        f.setAlwaysOnTop(true);
                    }
                    catch(SecurityException se)
                    {
                        se.printStackTrace();
                    }
                    About about = new About(f);
                    f.setContentPane(about);
                    f.pack();
                    Dimension center = WindowUtil.centerWindow(host, f);
                    f.setLocation(center.width, center.height);
                    f.setVisible(true);
                }
                catch (IOException ioe)
                {
                    WindowUtil.printExceptionError(ioe.getCause(), ioe.getMessage(), true);
                    ioe.printStackTrace();
                }
            }
        }); // End anonymous
    }
} // End class
